// ©2014 The Telos Alliance. All Rights Reserved.

// codec.js: jQuery-based codec sliders for Z1
// author:   Jeff Bencin <jeff.bencin@gmail.com>
//
// REQUIRES:
//   jquery.js
//   jquery-ui.js
//   jquery-ui.css
//   common.js
//   common.css
//   codec.css
//
// Javascript namespace: csl_*
// CSS namespace: csl-* (CSS in file codec.css)
//
// Interface:
//   Page containing this script must have div or span selectable by csl_rootSelector
//   This element must contain the following data-* parameters:
//     codecs:           JSON object containing array of name/value pairs of available codecs
//     selected-codec:   JSON object name/value pair
//     selected-mode:    JSON object name/value pair
//     selected-maxbr:   JSON object name/value pair
//     selected-minbr:   JSON object name/value pair
//     selected-maxbuf:  JSON object name/value pair
//     selected-minbuf:  JSON object name/value pair
//     name-profile:     string, name of "codec profile" input field

// XXX: csl_data_* variables contain data about the available options on the Z1. Some are hardcoded
//      and some are gotten via AJAX (those initialized to null).

// Data on bitrates for all codecs
// XXX: Only set via csl_updateVarDataCodec()
var csl_data_codec = null;
// Data on codec profiles
var csl_data_profile = null;
// Value of "User setting" codec profile
var csl_data_profile_custom = "0";
// Map channel mode indices to values
var csl_data_mode = ["stereo", "mono"];
// Min and max buffer times available
var csl_data_buftime = [
	{value:      "0", text:   "0 ms"},
	{value:     "20", text:  "20 ms"},
	{value:     "50", text:  "50 ms"},
	{value:    "100", text: "100 ms"},
	{value:    "200", text: "200 ms"},
	{value:    "300", text: "300 ms"},
	{value:    "500", text: "500 ms"},
	{value:   "1000", text:  "1 sec"},
	{value:   "2000", text:  "2 sec"},
	{value:   "3000", text:  "3 sec"},
	{value:   "5000", text:  "5 sec"},
	{value:   "8000", text:  "8 sec"},
	{value:  "11000", text: "11 sec"},
	{value:  "30000", text: "30 sec"},
	{value:  "60000", text: "60 sec"},
	{value: "300000", text:  "5 min"}
];
// Tooltip text
var csl_data_buffer_desc =
	"All codec modes offer automatic buffer management within the range set in this control." +
	" We suggest setting the Min buffer for 2x the expected receive packet jitter, and setting the Max buffer for 5x the expected packet jitter (or higher)." +
	" The ACT function will strive for the Min buffer as set while maintaining maximum successful packet reception." +
	" Set Min and Max buffer both to your desired buffer size to disable ACT buffer management.";
// Limits on Min and max buffer times. Use values in between legal ones to protect from floating point problems
var csl_data_min_maxbuf = 10;
var csl_data_max_minbuf = 6000;

// The selector of the element where everything will be done
var csl_rootSelector = "#jquerySectCodec";
// jQuery object conatining the "codec profiles" input
var csl_jq_profile = null;
// Index of all possible bitrate values, derived from csl_data_codec
var csl_bitrates = null;

// Called when document loads
$(document).ready(function() {
	var root = $(csl_rootSelector);
	var table, table2, tr, tr2, td;

	// Hide element to prevent reflow before we finish
	root.hide();
	root.empty();
	
	// Get parameters passed by CGI program
	var selCodec  = root.data("selected-codec");
	var selMode   = root.data("selected-mode");
	var selMinbr  = root.data("selected-minbr");
	var selMaxbr  = root.data("selected-maxbr");
	var selMinbuf = root.data("selected-minbuf");
	var selMaxbuf = root.data("selected-maxbuf");

	// Set up inputs for these values that will be submitted with the form
	$('<input type="hidden">').attr('id', 'csl-val_codec').attr('name', selCodec.name).val(selCodec.value).appendTo(root);
	$('<input type="hidden">').attr('id', 'csl-val_mode').attr('name', selMode.name).val(selMode.value).appendTo(root);
	$('<input type="hidden">').attr('id', 'csl-val_minbr').attr('name', selMinbr.name).val(selMinbr.value).appendTo(root);
	$('<input type="hidden">').attr('id', 'csl-val_maxbr').attr('name', selMaxbr.name).val(selMaxbr.value).appendTo(root);
	$('<input type="hidden">').attr('id', 'csl-val_minbuf').attr('name', selMinbuf.name).val(selMinbuf.value).appendTo(root);
	$('<input type="hidden">').attr('id', 'csl-val_maxbuf').attr('name', selMaxbuf.name).val(selMaxbuf.value).appendTo(root);

	// Make table of codec sliders
	$('<div>').addClass("csl-vertical_spacer").appendTo(root);
	table = $('<table>').addClass("com-wrapper csl-list").appendTo(root);

	// Make table headers
	tr = $('<tr>').addClass("com-bold").appendTo(table);
	$('<td>').text("Codecs").appendTo(tr);
	td = $('<td>').text("Bitrate range").addClass("com-center csl-tmpdisp_bitrates").appendTo(tr);
	td.data("default_text", td.text());

	// Make Row for each codec
	var codecs = root.data("codecs");
	for (var i in codecs) {
		var codec = codecs[i];
		tr = $('<tr>').addClass("csl-codec_container com-selectable").attr('title', "tooltip missing").appendTo(table);
		tr.data('value', codec.value);
		tr.data('props', codec.props);
		tr.data('usage', codec.usage);
		$('<td>').text(codecs[i].name).addClass("csl-label ui-corner-left").appendTo(tr);
		td = $('<td>').addClass("ui-corner-right").appendTo(tr);
		$('<div>').slider({
			disabled: true,
			range: true,
			min: 0
		}).appendTo(td);
	}

	// Draw buffer slider
	$('<div>').addClass("com-wrapper csl-vertical_spacer").appendTo(root);
	table = $('<table>').addClass("com-wrapper csl-list csl-buffer_table").appendTo(root);
	tr = $('<tr>').addClass("com-bold").appendTo(table);
	$('<td>').addClass("com-center").text("Buffer range").appendTo(tr);
	tr = $('<tr>').addClass("csl-buffer_container").attr('title', csl_data_buffer_desc).appendTo(table);
	td = $('<td>').appendTo(tr);
	$('<div>').addClass("csl-buffer_slider").slider({
		disabled: true,
		range: true,
		min: 0,
		max: csl_data_buftime.length-1,
		slide: csl_onSlideBuftime
	}).appendTo(td);

	// Table for status and channel mode buttons
	$('<div>').addClass("csl-vertical_spacer").appendTo(root);
	table = $('<table>').addClass("com-wrapper").appendTo(root);
	tr = $('<tr>').appendTo(table);

	// Mono and Stereo buttons
	td = $('<td>').appendTo(tr);
	table2 = $('<table>').addClass("com-wrapper com-radio csl-mode").appendTo(td);
	tr2 = $('<tr>').appendTo(table2);
	$('<td>').addClass("com-selectable ui-corner-left").data('value', '1').click(csl_onClickChannelMode).text(csl_data_mode[1].toUpperCase()).appendTo(tr2);
	$('<td>').addClass("com-selectable ui-corner-right").data('value', '0').click(csl_onClickChannelMode).text(csl_data_mode[0].toUpperCase()).appendTo(tr2);

	// Make table to display selected settings
	td = $('<td>').appendTo(tr);
	table2 = $('<table>').addClass("com-wrapper com-status").appendTo(td);
	tr2 = $('<tr>').appendTo(table2);
	$('<td>').addClass("com-label ui-corner-left").text("Codec").appendTo(tr2);
	$('<td colspan=4>').addClass("com-value csl-disp_codec ui-corner-right").appendTo(tr2);
	tr2 = $('<tr>').appendTo(table2);
	$('<td>').addClass("com-label ui-corner-left").text("Min. bitrate").appendTo(tr2);
	$('<td>').addClass("com-value csl-disp_minbr ui-corner-right").appendTo(tr2);
	$('<td>').addClass("com-gap").appendTo(tr2);
	$('<td>').addClass("com-label ui-corner-left").text("Min. buffer").appendTo(tr2);
	$('<td>').addClass("com-value csl-disp_minbuf ui-corner-right").appendTo(tr2);
	tr2 = $('<tr>').appendTo(table2);
	$('<td>').addClass("com-label ui-corner-left").text("Max. bitrate").appendTo(tr2);
	$('<td>').addClass("com-value csl-disp_maxbr ui-corner-right").appendTo(tr2);
	$('<td>').addClass("com-gap").appendTo(tr2);
	$('<td>').addClass("com-label ui-corner-left").text("Max. buffer").appendTo(tr2);
	$('<td>').addClass("com-value csl-disp_maxbuf ui-corner-right").appendTo(tr2);

	// Generate tooltips
	$(".csl-codec_container").tooltip({
		position: {
			my: "left top",
			at: "right top",
		},
		content: function() {
			var selMode   = $("#csl-val_mode").val();
			var codecIdx = $(this).data('value');
			var codec = csl_data_codec[csl_data_mode[selMode]][codecIdx];
			var bitrates = codec.bitrates;
			var bitrateText = bitrates[0] + " Kbps";
			var lastBr = bitrates.length-1
			if (lastBr > 0) {
				bitrateText += " - " + bitrates[lastBr] + " Kbps";
			}

			var th = $(this);
			var container = $('<span>');
			$('<span>').addClass("com-bold").text("Available bitrates: ").appendTo(container);
			$('<span>').text(bitrateText).appendTo(container);
			var text = th.data("props");
			if (text) {
				$('<br>').appendTo(container);
				$('<br>').appendTo(container);
				$('<span>').addClass("com-bold").text("Properties").appendTo(container);
				$('<br>').appendTo(container);
				$('<span>').text(text).appendTo(container);
			}
			text = th.data("usage");
			if (text) {
				$('<br>').appendTo(container);
				$('<br>').appendTo(container);
				$('<span>').addClass("com-bold").text("Suggested Use").appendTo(container);
				$('<br>').appendTo(container);
				$('<span>').text(text).appendTo(container);
			}
			return container;
		}
	});
	$(".csl-buffer_container").tooltip({
		position: {
			my: "left top",
			at: "right top",
		},
	});

	// Apply changes
	root.show();

	// Get codec data
	$.getJSON("json/bitrates", function(data) {
		csl_data_codec = data;
		csl_bitrates = com_parseBitratesFromJSON(data);
		csl_updateDisplay();
	});

	// Get preset codec profiles
	$.getJSON("json/profiles", function(data) {
		csl_data_profile = data;
		csl_jq_profile = $("[name='" + root.data("name-profile") + "']");
		csl_jq_profile.on('change', csl_onChangeProfile);
	});
});

// Callback function. Called when codec profile changes
function csl_onChangeProfile() {
	var newProf = csl_data_profile[$(this).val()];
	if (!newProf) return;

	// Have to check codec bitrates first, might be out of allowed range for channel mode
	var newCodec = newProf.codec;

	// XXX: This is a hack to work around the fact that current profiles do not support mono.
	// FIXME ASAP!!!
	var newMode  = 0;
	var selMode  = $("#csl-val_mode").val();
	var bitrates = csl_data_codec[csl_data_mode[newMode]][newCodec].bitrates;
	var newMinbr = Math.max(bitrates[0], newProf.minbr);
	var newMaxbr = Math.min(bitrates[bitrates.length-1], newProf.maxbr);

	// Set new values on form
	$("#csl-val_codec").val(newCodec);
	$("#csl-val_mode").val(newMode);
	$("#csl-val_minbr").val(newMinbr);
	$("#csl-val_maxbr").val(newMaxbr);
	$("#csl-val_minbuf").val(newProf.minbuf);
	$("#csl-val_maxbuf").val(newProf.maxbuf);

	csl_updateDisplay();
}

// Callback function. Set selected channel mode (stereo or mono)
function csl_onClickChannelMode() {
	// Determine new values
	var selCodec = $("#csl-val_codec").val();
	var newMode = $(this).data("value");
	var bitrates = csl_data_codec[csl_data_mode[newMode]][selCodec].bitrates;
	var newMinbr = bitrates[0];
	var newMaxbr = bitrates[bitrates.length-1];

	// Set new values on form
	$("#csl-val_mode").val(newMode);
	$("#csl-val_minbr").val(newMinbr);
	$("#csl-val_maxbr").val(newMaxbr);

	// Show "custom setting" for profile
	csl_jq_profile.val(csl_data_profile_custom);

	csl_updateDisplay();
}

// Callback function. Set selected codec
function csl_onClickCodecRow() {
	var range = $(this).find(".ui-slider").slider("values");

	// Get values for new codec
	var newCodec = $(this).data("value");
	var newMinbr = csl_bitrates[range[0]];
	var newMaxbr = csl_bitrates[range[1]];

	// Set new values on form, reset bitrate ranges to min/max
	$("#csl-val_codec").val(newCodec);
	$("#csl-val_minbr").val(newMinbr);
	$("#csl-val_maxbr").val(newMaxbr);

	// Show "custom setting" for profile
	csl_jq_profile.val(csl_data_profile_custom);

	csl_updateDisplay();
}

function csl_onMouseenterCodecRow() {
	var selMode   = $("#csl-val_mode").val();
	var codec     = $(this).data("value");
	var codecName = $(this).find(".csl-label").text();
	var bitrates  = csl_data_codec[csl_data_mode[selMode]][codec].bitrates;

	$(".csl-disp_codec").addClass("com-highlighted_text").text(codecName)
	$(".csl-disp_minbr").addClass("com-highlighted_text").text(bitrates[0] + " Kbps");
	$(".csl-disp_maxbr").addClass("com-highlighted_text").text(bitrates[bitrates.length-1] + " Kbps");
}

function csl_onMouseleaveCodecRow() {
	// Don't call csl_updateDisplay(), that would be slow
	var selCodec  = $("#csl-val_codec").val();
	var selMinbr  = $("#csl-val_minbr").val();
	var selMaxbr  = $("#csl-val_maxbr").val();
	var selCodecName;

	$(".csl-codec_container").each(function() {
		var codecId = $(this).data("value");

		if (codecId==selCodec) {
			selCodecName = $(this).find(".csl-label").text();
			return false;
		}	
	});

	$(".csl-disp_codec").removeClass("com-highlighted_text").text(selCodecName)
	$(".csl-disp_minbr").removeClass("com-highlighted_text").text(selMinbr + " Kbps");
	$(".csl-disp_maxbr").removeClass("com-highlighted_text").text(selMaxbr + " Kbps");
}

// Callback function. Set values when buffer slider is moved
function csl_onSlideBuftime(event, ui) {
	var minbuf = csl_data_buftime[ui.values[0]].value;
	var maxbuf = csl_data_buftime[ui.values[1]].value;

	if (parseInt(minbuf) > csl_data_max_minbuf) return false;
	if (parseInt(maxbuf) < csl_data_min_maxbuf) return false;

	$("#csl-val_minbuf").val(minbuf);
	$("#csl-val_maxbuf").val(maxbuf);

	// Show "custom setting" for profile
	csl_jq_profile.val(csl_data_profile_custom);

	csl_updateDisplay();
}

// Add event handler to stop movement to bitrates it does not support
// Have to do this outside csl_updateSliders() so closures work
function csl_makeOnSlideBitrate(bitrates) {
	return function(event, ui) {
		// Get actual bitrate values
		var minbr = csl_bitrates[ui.values[0]];
		var maxbr = csl_bitrates[ui.values[1]];

		// Test if bitrate values are legal
		if (bitrates) {
			if (bitrates.indexOf(minbr) < 0) return false;
			if (bitrates.indexOf(maxbr) < 0) return false;
		}

		// Set new value on form
		$("#csl-val_minbr").val(minbr);
		$("#csl-val_maxbr").val(maxbr);

		csl_updateDisplay();
	};
}

// Show updates and actions to users
function csl_updateDisplay() {
	var root = $(csl_rootSelector);

	var selMode   = $("#csl-val_mode").val();
	var selCodec  = $("#csl-val_codec").val();
	var selMinbr  = $("#csl-val_minbr").val();
	var selMaxbr  = $("#csl-val_maxbr").val();
	var selMinbuf = $("#csl-val_minbuf").val();
	var selMaxbuf = $("#csl-val_maxbuf").val();

	var allCodecs   = csl_data_codec[csl_data_mode[selMode]];
	var allBitrates = csl_bitrates;

	var selCodecName;
	var sliderSize  = allBitrates.length - 1;

	// Update slider ranges, highlight and enable selected one
	$(".csl-codec_container").each(function() {
		var row = $(this);
		var sl = row.find(".ui-slider");
		var codecId = row.data("value");
		var codecBitrates = allCodecs[codecId].bitrates;
		var cbLastIdx = codecBitrates.length - 1;

		var minbr;
		var maxbr;

		row.off("click");
		row.off("hover");
		sl.off("slide");

		if (codecId==selCodec) {
			// If this is the selected codec, use selected range
			row.addClass("com-selected");
			sl.slider("enable");
			sl.on("slide", csl_makeOnSlideBitrate(codecBitrates));
			minbr = parseInt(selMinbr);
			maxbr = parseInt(selMaxbr);
			selCodecName = row.find(".csl-label").text();
		} else {
			// If this is not selected codec, use min/max bitrates and disable slider
			row.removeClass("com-selected");
			row.click(csl_onClickCodecRow)
			// Putting this information in tooltip instead for now...
			//row.hover(csl_onMouseenterCodecRow, csl_onMouseleaveCodecRow);
			sl.slider("disable");
			minbr = codecBitrates[0];
			maxbr = codecBitrates[cbLastIdx];
		}

		// Find range indices for bitrates
		var rangeMin = allBitrates.indexOf(minbr);
		var rangeMax = allBitrates.indexOf(maxbr);

		// Update slider
		sl.slider("values", [rangeMin, rangeMax]);
		sl.slider("option", "max", sliderSize);
	});

	// Update buffer slider
	var buftime = csl_data_buftime;
	var rangeMin;
	var rangeMax;
	var selMinbufText = selMinbuf;
	var selMaxbufText = selMaxbuf;

	for (var i in buftime) {
		var value = buftime[i].value;
		if (value==selMinbuf) {
			rangeMin = i;
			selMinbufText = buftime[i].text;
		}
		if (value==selMaxbuf) {
			rangeMax = i;
			selMaxbufText = buftime[i].text;
		}
	}
	if (rangeMin && rangeMax) {
		$(".csl-buffer_slider").slider("values", [rangeMin, rangeMax]).slider("enable");
		$(".csl-buffer_table").show();
	} else {
		// We got unrecognized buffer values, disable to be safe
		$(".csl-buffer_table").hide();
	}

	// Highlight correct mode button
	com_selectByValue($(".csl-mode").find("td"), selMode);

	// Remove any changes from mouseovers
	$(".com-highlighted_text").each(function() {
		var me = $(this);
		var defText = me.data("default_text");
		me.removeClass("com-highlighted_text");
		me.text(defText);
	});

	// Update status displays
	$(".csl-disp_codec").text(selCodecName)
	$(".csl-disp_minbr").text(selMinbr + " Kbps");
	$(".csl-disp_maxbr").text(selMaxbr + " Kbps");
	$(".csl-disp_minbuf").text(selMinbufText);
	$(".csl-disp_maxbuf").text(selMaxbufText);
}

