// ©2014 The Telos Alliance. All rights reserved.
//
// autodial.js: jQuery-based replacement for Java autodial app
// Author:      Jeff Bencin <jeff.bencin@gmail.com>
//
// REQUIRES:
//   jquery.js
//   common.js
//   common.css
//   congraph.js (To update stats)
//   congraph.css
//   autodial.css
//   meters.js (If meters are desired)
//
// Javascript namespace: adl_*
// CSS namespace: adl-* (CSS in file autodial.css)

// Make sure console.log does not crash in IE
if(!window.console) { window.console = {log: function(){} }; }

// The selector of the element where everything will be done
var adl_rootSelector = '#jquerySectAutodial';

// State variables
var adl_valDialmode = 'book';

// Request to the ZIP server
var adl_serverRequest;
var adl_serverTimeout;

// Called when document loads
$(document).ready(function() {
	// Configure jqGrid library
	$.extend($.jgrid.defaults, {
		datatype:    "local",
		gridview:    true,
		viewrecords: true,
		rowNum:      10,
		height:      "auto",
		width:       458,
		sortable:    true
	});

	$.extend($.jgrid.edit, {
		bSubmit:     "Save",
		resize:      false,
		closeAfterEdit: true,
		viewPagerButtons: false
	});

	$.extend($.jgrid.view, {
		resize:      false
	});

	$.extend($.jgrid.nav, {
		addtitle:    "Add new item",
		edittitle:   "Edit selected item",
		deltitle:    "Delete selected item",
		viewtitle:   "View selected item",
		viewicon:    "ui-icon-info",
		searchtitle: "Search",
		refreshtitle:"Refresh"
	});

	var root = $(adl_rootSelector);

	// Hide element to prevent reflow before we finish
	root.hide();
	root.empty();

	// Construct page
	var table, tr, td, div, p, sel, grid, pager, jqObj;
	var table2, tr2, td2, div2;
	var table3, tr3, td3;
	table = $('<table>').addClass("adl-maintable").appendTo(root);
	tr = $('<tr>').appendTo(table);

	// Add control buttons
	td = $('<td>').addClass("adl-control").appendTo(tr);
	table2 = $('<table>').addClass("com-wrapper com-radio adl-dialmode").appendTo(td);
	tr2 = $('<tr>').appendTo(table2);
	$('<td>').data('value', 'book').data('display', '.adl-disp-book').text("Phonebook").appendTo(tr2).click(adl_onClickGetPeers);
	$('<td>').data('value', 'srvr').data('display', '.adl-disp-srvr').text("Server").appendTo(tr2);
	$('<td>').data('value', 'hist').data('display', '.adl-disp-hist').text("History").appendTo(tr2).click(adl_onClickGetHistory);
	$('<td>').data('value', 'manl').data('display', '.adl-disp-manl').text("Manual").appendTo(tr2);
	tr2.children("td").addClass("com-selectable ui-corner-top").click(adl_onClickDialmode);

	// Phone book dial
	div  = $('<div>').addClass("adl-dial_sect adl-disp-book").attr('id', 'adl-phonebook').appendTo(td);
	grid = $("<table id='adl-phonebook_grid'>").appendTo(div);
	$("<div id='adl-phonebook_pager'>").appendTo(div);
	var protocolMenu = {
		"tscp" : "TSCP (server)",
		"tscpd" : "TSCP (direct)",
		"sip" : "SIP",
		"rtp" : "RTP",
		"" : "Other"
	};
	var protocolFormatter = function (val) { return protocolMenu[val]; }
	var panicMenu = {
		""  : "",
		"0" : "Disabled",
		"1" : "Bit 1",
		"2" : "Bit 2",
		"3" : "Bit 3",
		"4" : "Bit 4",
		"5" : "Bit 5",
		"6" : "Bit 6",
		"7" : "Bit 7",
		"8" : "Bit 8"
	};
	var panicFormatter = function (val) { return panicMenu[val]; }
	var redialMenu = {
		""   : "",
		"-1" : "Use global setting",
		"0"  : "None",
		"1"  : "Retry 3 times",
		"2"  : "Retry 10 times",
		"3"  : "Retry forever"
	};
	var redialFormatter = function (val) { return redialMenu[val]; }
	grid.jqGrid({
		colNames: ['ID', 'Alias', 'Protocol', 'Address/Name', 'Group', 'Group Password', 'Status', 'Panic Dial Bit', 'Redial Mode'],
		colModel: [
			{width:   0, name: 'id',       index: 'id',       key: true, hidden: true},
			{width: 235, name: 'alias',    index: 'alias',    editable: true, editrules: {edithidden:true}},
			{width:  80, name: 'protocol', index: 'protocol', editable: true, edittype: "select", editoptions: {value: protocolMenu}},
			{width:   0, name: 'name',     index: 'name',     editable: true, hidden: true, editrules: {edithidden:true, required:true}},
			{width:  80, name: 'group',    index: 'group',    editable: true},
			{width:   0, name: 'grpwd',    index: 'grpwd',    editable: true, hidden: true, editrules: {edithidden:true}},
			{width:  60, name: 'status',   index: 'status',   align: "center"},
			{width:   0, name: 'panic',    index: 'panic',    editable: true, edittype: "select", hidden: true, editrules: {edithidden:true}, editoptions: {value: panicMenu}, formatter: panicFormatter},
			{width:   0, name: 'redial',   index: 'redial',   editable: true, edittype: "select", hidden: true, editrules: {edithidden:true}, editoptions: {value: redialMenu}, formatter: redialFormatter}
		],
		deepempty: true,
		onSelectRow: adl_onSelectRow,
		pager: "#adl-phonebook_pager"
	});
	var editOpts = {
		mtype:      "GET",
		url:        "cmd/peers/add",
		editCaption: "Edit Peer",
		beforeShowForm: adl_beforeInitPeer,
		beforeSubmit: adl_beforeSubmitPeer,
		afterSubmit: adl_afterSubmitGeneric,
		afterComplete: adl_getPeers
	};
	var addOpts = {
		mtype:      "GET",
		url:        "cmd/peers/add",
		addCaption: "New Peer",
		beforeShowForm: adl_beforeInitPeer,
		beforeSubmit: adl_beforeSubmitPeer,
		afterSubmit: adl_afterSubmitGeneric,
		afterComplete: adl_getPeers
	};
	var delOpts = {
		mtype: "GET",
		url:   "cmd/peers/delete",
		beforeSubmit: adl_beforeSubmitGeneric,
		afterSubmit: adl_afterSubmitGeneric,
		afterComplete: adl_getPeers
	};
	grid.jqGrid('navGrid', "#adl-phonebook_pager", {
		edit: true,
		add: true,
		del: true,
		search: true,
		view: true,
		refresh: true,
		afterRefresh: adl_getPeers
	}, editOpts, addOpts, delOpts);

	// Server dial
	div = $('<div>').addClass("adl-dial_sect adl-disp-srvr").attr('id', 'adl-server').appendTo(td);

	// Search for groups
	table2 = $('<table id="adl-browse_groups">').addClass("com-wrapper adl-browse_server").appendTo(div);
	tr2 = $('<tr>').appendTo(table2);
	td2 = $('<td>').addClass("adl-header").appendTo(tr2);
	$('<span>').text("Search Filter").appendTo(td2);
	$('<input type="text" id="adl-val-group_filter">').val("*").appendTo(td2);
	$('<br>').appendTo(td2);
	$('<br>').appendTo(td2);
	$('<button id="adl-button-group_filter">').text("Search Groups").appendTo(td2).button({ icons: { primary: "ui-icon-search" } }).click(adl_onClickGroupSearch);
	$('<button id="adl-button-group_browse">').text("Open Group").appendTo(td2).button({ icons: { primary: "ui-icon-folder-open" }, disabled: true }).click(adl_onClickGroupOpen);
	td2 = $('<td>').appendTo(tr2);
	sel = $('<select id="adl-val-select_group" size="16">').data("toggle_button", "#adl-button-group_browse").change(adl_onChangeToggleButton).appendTo(td2);

	// Search group for devices
	table2 = $('<table id="adl-browse_devices">').addClass("com-wrapper adl-browse_server").hide().appendTo(div);
	tr2 = $('<tr>').appendTo(table2);
	td2 = $('<td>').addClass("adl-header").appendTo(tr2);
	$('<span>').text("Search Filter").appendTo(td2);
	$('<input type="text" id="adl-val-device_filter">').val("*").appendTo(td2);
	$('<br>').appendTo(td2);
	$('<br>').appendTo(td2);
	$('<span>').text("Group Password").appendTo(td2);
	$('<input type="text" id="adl-val-group_passwd">').appendTo(td2);
	$('<br>').appendTo(td2);
	$('<br>').appendTo(td2);
	$('<button id="adl-button-device_filter">').text("Search Devices").appendTo(td2).button({ icons: { primary: "ui-icon-search" } }).click(adl_onClickDeviceSearch);
	$('<button id="adl-button-device_save">').text("Save to Phonebook").appendTo(td2).button({ icons: { primary: "ui-icon-disk" }, disabled: true }).click(adl_onClickDeviceAddToPhonebook);
	$('<button id="adl-button-device_back">').text("Back").appendTo(td2).button({ icons: { primary: "ui-icon-arrowthick-1-w" } }).click(adl_onClickDeviceBack);
	td2 = $('<td>').appendTo(tr2);
	sel = $('<select id="adl-val-select_device" size="16">').data("toggle_button", "#adl-button-device_save").change(adl_onChangeToggleButton).appendTo(td2);

	// History dial
	div  = $('<div>').addClass("adl-dial_sect adl-disp-hist").attr('id', 'adl-history').appendTo(td);
	grid = $("<table id='adl-history_grid'>").appendTo(div);
	$("<div id='adl-history_pager'>").appendTo(div);
	grid.jqGrid({
		colNames: ['', 'Alias', 'Protocol', 'Address/Name', 'Remote unit', 'Group', 'Group Password', 'Status', 'Panic Dial Bit', 'Redial Mode'],
		colModel: [
			{width:  30, name: 'direction',     index: 'direction',     formoptions: {label:"Direction"}},
			{width:   0, name: 'peer.alias',    index: 'peer.alias',    editable: true, hidden: true, editrules: {edithidden:true}},
			{width:  80, name: 'peer.protocol', index: 'peer.protocol', edittype: "select", editable: true, editoptions: {value: protocolMenu}},
			{width:   0, name: 'peer.name',     index: 'peer.name',     editable: true, hidden: true, editrules: {edithidden:true, required:true}},
			{width: 204, name: 'peer.label',    index: 'peer.label',    viewable: false},
			{width:  80, name: 'peer.group',    index: 'peer.group',    editable: true},
			{width:   0, name: 'peer.grpwd',    index: 'peer.grpwd',    viewable: false, editable: true, hidden: true, editrules: {edithidden:true}},
			{width:  60, name: 'peer.status',   index: 'peer.status',   align: "center"},
			{width:   0, name: 'peer.panic',    index: 'panic',         viewable: false, edittype: "select", editable: true, hidden: true, editrules: {edithidden:true}, editoptions: {value: panicMenu}},
			{width:   0, name: 'peer.redial',   index: 'redial',        viewable: false, edittype: "select", editable: true, hidden: true, editrules: {edithidden:true}, editoptions: {value: redialMenu}},
		],
		deepempty: true,
		onSelectRow: adl_onSelectRow,
		pager: "#adl-history_pager"
	});
	var saveOpts = {
		mtype:       "GET",
		url:         "cmd/peers/add",
		editCaption: "Save to Phonebook",
		beforeShowForm: adl_beforeInitPeer,
		beforeSubmit: adl_beforeSubmitPeer,
		afterSubmit: adl_afterSubmitGeneric,
		afterComplete: adl_getHistory
	};
	grid.jqGrid('navGrid', "#adl-history_pager", {
		edit: true,
		editicon: "ui-icon-disk",
		edittitle: "Save to Phonebook",
		add: false,
		del: false,
		search: true,
		view: true,
		refresh: true,
		afterRefresh: adl_getHistory
	}, saveOpts);
	grid.jqGrid('navButtonAdd', "#adl-history_pager", {
		caption:       "",
		title:         "Clear History",
		buttonicon:    "ui-icon-trash",
		onClickButton: function() { $("#adl-dialog-clear_history").dialog("open"); },
		position:      "last"
	});

	// Manual dial
	div = $('<div>').addClass("adl-dial_sect adl-disp-manl").appendTo(td);
	$('<br>').appendTo(div);
	table2 = $('<table>').addClass("com-fullwidth").appendTo(div);
	tr2 = $('<tr>').addClass("adl-header").appendTo(table2);
	td2 = $('<td class="adl-proto_select">').text("Protocol").appendTo(tr2);
	$('<br>').appendTo(td2);

	// Select
	sel = $('<select>').attr('id', 'adl-val-protocol').on('change', adl_onChangeProtocol).appendTo(td2);
	$('<option>').data('display', '.adl-disp-ngrp').data('proto', 'tscp').text("TSCP (server)").appendTo(sel);
	$('<option>').data('display', '.adl-disp-addr').data('proto', 'tscpd').text("TSCP (direct)").appendTo(sel);
	$('<option>').data('display', '.adl-disp-nsrv').data('proto', 'sip').text("SIP (server)").appendTo(sel);
	$('<option>').data('display', '.adl-disp-addr').data('proto', 'sip').text("SIP (direct)").appendTo(sel);
	$('<option>').data('display', '.adl-disp-addr').data('proto', 'rtp').text("RTP").appendTo(sel);
	$('<option>').data('display', '.adl-disp-trgt').text("Other").appendTo(sel);

	// Text fields
	td2 = $('<td>').addClass("adl-disp-ngrp adl-disp-nsrv").text("Device Name").appendTo(tr2);
	$('<br>').appendTo(td2);
	$('<input>').attr('type', 'text').attr('id', 'adl-val-name').appendTo(td2);
	td2 = $('<td>').addClass("adl-disp-ngrp adl-disp-nsrv").appendTo(tr2); // This cell just displays '@'
	$('<br>').appendTo(td2);
	$('<span>').text("@").appendTo(td2);
	td2 = $('<td>').addClass("adl-disp-nsrv").text("Server").appendTo(tr2);
	$('<br>').appendTo(td2);
	$('<input>').attr('type', 'text').attr('id', 'adl-val-srvr').appendTo(td2);
	td2 = $('<td>').addClass("adl-disp-ngrp").text("Group").appendTo(tr2);
	$('<br>').appendTo(td2);
	$('<input>').attr('type', 'text').attr('id', 'adl-val-grup').appendTo(td2);
	td2 = $('<td>').addClass("adl-disp-addr").text("Address").appendTo(tr2);
	$('<br>').appendTo(td2);
	$('<input>').attr('type', 'text').attr('id', 'adl-val-addr').appendTo(td2);
	td2 = $('<td>').addClass("adl-disp-addr").text("Port").appendTo(tr2);
	$('<br>').appendTo(td2);
	$('<input>').attr('type', 'text').attr('id', 'adl-val-port').attr('maxlength', '5').attr('size', '5').appendTo(td2);
	td2 = $('<td>').addClass("adl-disp-trgt").text("Target").appendTo(tr2);
	$('<br>').appendTo(td2);
	$('<input>').attr('type', 'text').attr('id', 'adl-val-trgt').appendTo(td2);

	// Call and Disconnect buttons
	div = $('<div>').attr('id', 'adl-div-connect').addClass("com-wrapper").appendTo(td);
	div2 = $('<div>').data("value", "conn").appendTo(div);
	$('<button>').addClass("adl-connect").text("Connect").appendTo(div2);
	div2 = $('<div>').data("value", "disc").appendTo(div);
	$('<button>').addClass("adl-disconnect").appendTo(div2);
	div2 = $('<div>').data("value", "rtpa").appendTo(div);
	$('<button>').addClass("adl-connect").text("Add RTP Stream").data('notify', true).appendTo(div2);
	$('<button>').addClass("adl-disconnect").appendTo(div2);
	div.children("div").addClass("com-wrapper");
	jqObj = div.find("button").addClass("com-noselect ui-corner-all");
	jqObj.filter(".adl-connect").click(adl_onClickConnect);
	jqObj.filter(".adl-disconnect").text("Disconnect").click(adl_onClickDisconnect);

	// Display meters
	td = $('<td>').addClass("adl-meters").appendTo(tr);
	$('<div>').addClass("com-wrapper").attr('id', 'adl-meter_sect').appendTo(td);

	// Display status
	td = $('<td>').addClass("adl-status").appendTo(tr);
	table2 = $('<table>').addClass("com-status").appendTo(td);
	tr2 = $('<tr>').appendTo(table2);
	$('<td colspan=3>').addClass("com-title ui-corner-top-left").text("Connection").appendTo(tr2);
	tr2 = $('<tr>').appendTo(table2);
	$('<td>').addClass("com-label ui-corner-top-left").text("Status").appendTo(tr2);
	$('<td colspan=2>').addClass("com-value ui-corner-top-right cgr-disp-call_stat").appendTo(tr2);
	tr2 = $('<tr>').appendTo(table2);
	$('<td>').addClass("com-label").text("Peer").appendTo(tr2);
	$('<td colspan=2>').addClass("com-value cgr-disp-call_peer").appendTo(tr2);
	tr2 = $('<tr>').appendTo(table2);
	$('<td>').addClass("com-label ui-corner-bottom-left").text("Duration").appendTo(tr2);
	$('<td colspan=2>').addClass("com-value ui-corner-bottom-right cgr-disp-call_time").appendTo(tr2);
	tr2 = $('<tr>').appendTo(table2);
	$('<td>').addClass("com-empty").appendTo(tr2);
	$('<td>').addClass("com-empty").appendTo(tr2);
	$('<td>').addClass("com-empty").appendTo(tr2);
	tr2 = $('<tr>').appendTo(table2);
	$('<td>').addClass("com-title ui-corner-top-left").text("Frames").appendTo(tr2);
	$('<td>').addClass("com-label com-center").text("Receive").appendTo(tr2);
	$('<td>').addClass("com-label com-center ui-corner-top-right").text("Transmit").appendTo(tr2);
	tr2 = $('<tr>').appendTo(table2);
	$('<td>').addClass("com-label").text("Total").appendTo(tr2);
	$('<td>').addClass("com-value cgr-disp-rcv_pkts").appendTo(tr2);
	$('<td>').addClass("com-value cgr-disp-xmt_pkts ").appendTo(tr2);
	tr2 = $('<tr>').appendTo(table2);
	$('<td>').addClass("com-label").text("Buffered").appendTo(tr2);
	$('<td>').addClass("com-value cgr-disp-rcv_buff").appendTo(tr2);
	$('<td>').addClass("com-value cgr-disp-xmt_buff").appendTo(tr2);
	tr2 = $('<tr>').appendTo(table2);
	$('<td>').addClass("com-label").text("Min. Buffer").appendTo(tr2);
	$('<td>').addClass("com-value cgr-disp-rcv_mbuf").appendTo(tr2);
	$('<td>').addClass("com-value cgr-disp-xmt_mbuf").appendTo(tr2);
	tr2 = $('<tr>').appendTo(table2);
	$('<td>').addClass("com-label").text("Dropped").appendTo(tr2);
	$('<td>').addClass("com-value cgr-disp-rcv_drop").appendTo(tr2);
	$('<td>').addClass("com-value cgr-disp-xmt_drop").appendTo(tr2);
	tr2 = $('<tr>').appendTo(table2);
	$('<td>').addClass("com-label").text("Lost").appendTo(tr2);
	$('<td>').addClass("com-value cgr-disp-rcv_lost").appendTo(tr2);
	$('<td>').addClass("com-value cgr-disp-xmt_lost").appendTo(tr2);
	tr2 = $('<tr>').appendTo(table2);
	$('<td>').addClass("com-title").text("Audio").appendTo(tr2);
	$('<td>').addClass("com-label com-center").text("Receive").appendTo(tr2);
	$('<td>').addClass("com-label com-center").text("Transmit").appendTo(tr2);
	tr2 = $('<tr>').appendTo(table2);
	$('<td>').addClass("com-label").text("Codec").appendTo(tr2);
	$('<td>').addClass("com-value cgr-disp-rcv_codc").appendTo(tr2);
	$('<td>').addClass("com-value cgr-disp-xmt_codc").appendTo(tr2);
	tr2 = $('<tr>').appendTo(table2);
	$('<td>').addClass("com-label").text("Channels").appendTo(tr2);
	$('<td>').addClass("com-value cgr-disp-rcv_chan").appendTo(tr2);
	$('<td>').addClass("com-value cgr-disp-xmt_chan").appendTo(tr2);
	tr2 = $('<tr>').appendTo(table2);
	$('<td>').addClass("com-label").text("Sample Rate (Hz)").appendTo(tr2);
	$('<td>').addClass("com-value cgr-disp-rcv_srat").appendTo(tr2);
	$('<td>').addClass("com-value cgr-disp-xmt_srat").appendTo(tr2);
	tr2 = $('<tr>').appendTo(table2);
	$('<td>').addClass("com-label ui-corner-bottom-left").text("Bitrate (bits/s)").appendTo(tr2);
	$('<td>').addClass("com-value cgr-disp-rcv_bitr").appendTo(tr2);
	$('<td>').addClass("com-value cgr-disp-xmt_bitr ui-corner-bottom-right").appendTo(tr2);

	// Dialog boxes
	// Clear history
	div = $('<div id="adl-dialog-clear_history" title="Clear history">');
	p   = $('<p>').text("Clear entire call history?").appendTo(div);
	$('<span style="float:left; margin:0 7px 20px 0;">').addClass("ui-icon ui-icon-alert").appendTo(p);

	div.dialog({
		autoOpen:  false,
		resizable: false,
		modal:     true,
		height:    "auto",
		width:     "auto",
		buttons: [
			{
				text: "Clear",
				click: function() {
					adl_cmdHistoryClear();
					$(this).dialog("close");
				},
				icons: {
					primary: "ui-icon-trash"
				}
			},
			{
				text: "Cancel",
				click: function() {
					$(this).dialog("close");
				},
				icons: {
					primary: "ui-icon-close"
				}
			}
		]
	});

	// Generic dialog w/ close button. Make sure to set text before using!!!
	div = $('<div id="adl-dialog-generic" title="Notice">');

	div.dialog({
		autoOpen:  false,
		resizable: false,
		modal:     true,
		height:    "auto",
		width:     "auto",
		buttons: [
			{
				text: "Close",
				click: function() {
					$(this).dialog("close");
				},
				icons: {
					primary: "ui-icon-close"
				}
			}
		]
	});

	// Apply changes
	adl_updateDisplay();
	root.show();

	// Draw meters
	try {
		meters_init('adl-meter_sect', 'vertical', 53);
	} 
	catch (err) {
		console.log("Loading meters failed: " + err);
		$('#adl-meter_sect').remove();
	}

	// Subscribe to stats
	com_ajaxSubscribe("json/connection_stats", 1000, function(data) {
		adl_updateCallButton(data.call.idle);
	});

	// Initialize phonebook
	adl_getPeers();
});

// --------------------------------
// Functions used as event handlers
// --------------------------------

function adl_onChangeProtocol() {
	adl_updateDisplay();
}

function adl_onChangeToggleButton() {
	var selector = $(this).data('toggle_button');
	var action = "disable";

	// If this form has a value selected, enable button
	if ($(this).val()) {
		action = "enable";
	}
	$(selector).button(action);
	//console.log("adl_onChangeToggleButton(): " + selector + " -> " + action);
}

function adl_onClickDialmode() {
	adl_valDialmode = $(this).data('value');
	adl_updateDisplay();
}

function adl_onClickGetHistory() {
	grid = $("#adl-history_grid").resetSelection();
	return adl_getHistory(); // Strip out args from callback
}

function adl_onClickGetPeers() {
	grid = $("#adl-phonebook_grid").resetSelection();
	return adl_getPeers(); // Strip out args from callback
}

function adl_onClickGroupSearch() {
	adl_cmdGroupFilter();
}

function adl_onClickGroupOpen() {
	$("#adl-browse_groups").hide();
	$("#adl-browse_devices").show();
	adl_cmdDeviceFilter();
}

function adl_onClickDeviceSearch() {
	adl_cmdDeviceFilter();
}

function adl_onClickDeviceBack() {
	$("#adl-browse_devices").hide();
	$("#adl-browse_groups").show();
}

function adl_onClickDeviceAddToPhonebook() {
	var args = new Object();
	var peer = new Object();

	peer.protocol = "tscp";
	peer.group    = $("#adl-val-select_group").val();
	peer.name     = $("#adl-val-select_device").val();

	// These will be the GET params
	args.addr  = adl_makeAddr(peer);
	args.alias = peer.name;
	args.grpwd = $("#adl-val-group_passwd").val();

	// Create request URL
	var url = com_appendGETParams("cmd/peers/add", args);

	console.log("AJAX command: " + url);

	// Show dialog window after submitting request and getting response
	$.ajax(url).success(function(data) {
		var msg = data;
		if (msg === "OK") {
			msg = "Device added to phonebook";
		}
		$("#adl-dialog-generic").text(msg).dialog("open");
	});
}

function adl_onClickDisconnect() {
	var div = $('#adl-div-connect');
	var disabled = div.data('disabled');
	if (disabled) return;
	adl_cmdDisconnect();
	div.data('disabled', true);
}

function adl_onClickConnect() {
	var div = $('#adl-div-connect');
	var disabled = div.data('disabled');
	if (disabled) return;
	var notify = $(this).data('notify');
	adl_cmdConnect(notify);
	div.data('disabled', true);
}

function adl_onSelectRow() {
	// We have to do this AFTER the function calling this callback finishes
	setTimeout(adl_updateCallButton, 1);
}

// -------------------------------------
// Functions to issue commands to server
// -------------------------------------

function adl_cmdDeviceFilter() {
	var args = new Object();
	args.sid    = Date.now();
	args.group  = $("#adl-val-select_group").val();
	args.filter = $("#adl-val-device_filter").val();
	args.grpwd  = $("#adl-val-group_passwd").val();
	var url = com_appendGETParams("cmd/server/device", args);

	console.log("AJAX command: " + url);

	var sel = $("#adl-val-select_device");
	adl_getIncrementalJSON(url, sel);
}

function adl_cmdGroupFilter() {
	var args = new Object();
	args.sid    = Date.now();
	args.filter = $("#adl-val-group_filter").val();
	var url = com_appendGETParams("cmd/server/group", args);

	console.log("AJAX command: " + url);

	var sel = $("#adl-val-select_group");
	adl_getIncrementalJSON(url, sel);
}

function adl_cmdHistoryClear() {
	var url = "cmd/history/clear";

	console.log("AJAX command: " + url);
	$.ajax(url).success(function() {
		var args = new Object();
		args.sid   = Date.now();
		args.force = "true";
		adl_getHistory(args);
	});
}

function adl_cmdDisconnect() {
	var args = new Object();
	args.sid = Date.now();

	var url = com_appendGETParams("cmd/call/disconnect", args);

	console.log("AJAX command: " + url);
	$.ajax(url).success(function() {
		//adl_updateCallButton(false);
	});
}

function adl_cmdConnect(notify) {
	var args = new Object();
	var peer = adl_selectedPeer();
	if (!peer) return;

	// Create request URL
	args.sid  = Date.now();
	args.addr = adl_makeAddr(peer);
	var url = com_appendGETParams("cmd/call/connect", args);

	if (notify) {
		var msg = "Added stream '" + args.addr + "'";
		$("#adl-dialog-generic").text(msg).dialog("open");
	}

	// Send request to server
	console.log("AJAX command: " + url);
	$.ajax(url).success(function() {
		//adl_updateCallButton(true);
	});
}

// -----------------------------------
// Functions to fetch data from server
// -----------------------------------

// Receive JSON data in partial increments from AJAX request
//   url: Command encoded as a GET request
//   sel: jQuery object for select element where results will be placed
function adl_getIncrementalJSON(url, sel) {
	// We can only have one server request at a time
	var request = adl_serverRequest;
	request && request.abort();
	request = new XMLHttpRequest();

	// Clear options from last response, and make sure onchange handler is called
	sel.empty().trigger("change");

	// Close the request after 5 seconds of not recieving data
	var restartTimeout = function () {
		clearTimeout(adl_serverTimeout);
		adl_serverTimeout = setTimeout(onTimeout, 5000);
	};

	var onTimeout = function () {
		if (request) {
			//console.log(url + ": aborting, " + request.responseText.length + " bytes received");
			parseResponse(); // In case we still have unparsed data
			request.abort();
		}
		clearTimeout(adl_serverTimeout);
		adl_serverTimeout = null;
	};

	// Do this each time we receive data. Note that data may not be complete.
	var onProgress = function () {
		//console.log(url + ": onProgress() readyState=" + request.readyState);
		if (request.readyState < 3) return;
		parseResponse();
		if (request.readyState > 3) return;
		restartTimeout();
	};

	var responseIdx = 0;
	var parseResponse = function () {
		var unparsedText = request.responseText.substring(responseIdx);
		var parseIdx = 0; // Where we start parseJSON
		var sepIdx = 0;   // Where last separator was found

		while (true) {
			// Here we try to find a valid JSON object
			// The response consists of multiple JSON objects separated by '/n/n'
			// So we will expand the string with data until the next '/n/n' until parseJSON() succeeds
			var sepIdx = unparsedText.indexOf("\n\n", sepIdx);
			if (sepIdx < 0) break;

			var text = unparsedText.substring(parseIdx, sepIdx);

			// Search for next non-whitespace character. Use this as starting point for next '\n\n' search
			var c;
			var ws = /\s/m; // Regex for whitespace
			do {
				c = text.charAt(++sepIdx);
			} while (ws.test(c));

			// Try to parse text as a JSON object
			var json;
			try {
				json = $.parseJSON(text);
			} catch (err) {
				// JSON object is not complete, try again with more data
				console.log("JSON parse error: " + text);
				continue;
			}

			// Text was successfully parsed, move pointer by amount of text parsed
			responseIdx += sepIdx - parseIdx;

			// Next time, start after end of last valid JSON object
			parseIdx = sepIdx;

			// Add the items to the select element
			for (var i in json) {
				var item = json[i];
				var color = 'black';
				switch (item.status) {
					case -1: color = 'red';   break;
					case  1: color = 'green'; break;
				}
				$('<option>').text(item.name).css('color', color).appendTo(sel);
			}
		}
	};

	request.onprogress = onProgress;
	request.open("GET", url, true);
	request.send(null);

	adl_serverRequest = request;
	restartTimeout();
}

// Fetch phonebook entries from server and update display
function adl_getPeers(args) {
	args = args || new Object();
	args.sid = Date.now();
	var url = com_appendGETParams("json/peers", args);
	$.getJSON(url, function(data) {
		// Preprocessing
		for (var i in data) {
			adl_processPeer(data[i]);
		}
		// Update grid
		$("#adl-phonebook_grid").jqGrid('clearGridData').jqGrid('setGridParam', {data: data}).trigger("reloadGrid");
	});
}

// Fetch call history from server and update display
function adl_getHistory(args) {
	args = args || new Object();
	args.sid = Date.now();
	var url = com_appendGETParams("json/history", args);
	$.getJSON(url, function(data) {
		// Preprocessing
		for (var i in data) {
			adl_processPeer(data[i].peer);
		}
		// Update grid
		$("#adl-history_grid").jqGrid('clearGridData').jqGrid('setGridParam', {data: data}).trigger("reloadGrid");
	});
}

// ---------------------------
// Functions to update display
// ---------------------------

// Toggle big call button between "Connect" and "Disconnect"
function adl_updateCallButton(idle) {
	var callDiv = $('#adl-div-connect');

	// Determine if we are on a call or not
	if (idle === null || idle === undefined) {
		idle = callDiv.data('idle');
	} else {
		callDiv.data('idle', idle);
		callDiv.data('disabled', false);
	}

	// Determine which button set should be displayed
	var val;
	switch (idle) {
		case true:
			val = "conn";
			break;
		case false:
			// If we are on an RTP call, and user has selected another
			// RTP peer, allow user to add additional peers
			var peerConnected = $(".cgr-disp-call_peer").text();
			if (/^rtp:/.test(peerConnected)) {
				var peerSelected = adl_selectedPeer();
				if (peerSelected && (peerSelected.protocol === "rtp")) {
					val = "rtpa";
					break;
				}
			}
			val = "disc";
			break;
		default:
			val = "none";
			break;
	}

	// If we are already displaying the right thing, return
	if (val === callDiv.data('display')) return;

	var divs = callDiv.children("div");
	divs.hide();
	divs.each(function () {
		if (val === $(this).data('value')) {
			$(this).show();
		}
	});
	callDiv.data('display', val);
}

// Update all parts of display
function adl_updateDisplay() {
	var root = $(adl_rootSelector);

	var dialmodeButtons = $(".adl-dialmode").find("td");
	com_selectByValue(dialmodeButtons, adl_valDialmode);

	var protoSelect = $('#adl-val-protocol')
	com_htmlSelectByValue(protoSelect.find("option"), protoSelect.val());

	adl_updateCallButton();
}

// ---------------
// Misc. functions
// ---------------

// Return peer currently selected by user
function adl_selectedPeer() {
	var peer;

	// Get dialing mode from UI
	var dialmode = $(".adl-dialmode").find(".com-selected");
	switch (dialmode.data('value')) {
		case 'manl': // Manual dialing
			var selectProto = $('#adl-val-protocol').find(':selected'); // Get selected option
			var inputs = $(selectProto.data('display')).find('input'); // Get displayed input fields

			// Fill in peer object
			peer = new Object();
			peer.protocol = selectProto.data('proto');
			inputs.each(function() {
				var id  = $(this).attr('id');
				var val = $(this).val();
				if (!val.length) return;
				switch (id) {
					case 'adl-val-name':
					case 'adl-val-trgt':
					case 'adl-val-addr': peer.name = val; break;
					case 'adl-val-grup': peer.group = val; break;
					case 'adl-val-port': peer.port = val; break;
					case 'adl-val-srvr': peer.server = val; break;
				}
			});
			break;
		case 'book': // Phone book entry
			var grid = $("#adl-phonebook_grid");
			var id = grid.jqGrid('getGridParam','selrow');
			if (!id) return;
			peer = grid.jqGrid('getRowData', id);
			break;
		case 'hist': // History entry
			var grid = $("#adl-history_grid");
			var id = grid.jqGrid('getGridParam','selrow');
			if (!id) return;
			var row = grid.jqGrid('getRowData', id);
			peer = new Object();
			peer.protocol = row["peer.protocol"];
			peer.name     = row["peer.name"];
			peer.group    = row["peer.group"];
			break;
		case 'srvr': // Server dial
			peer = new Object();
			peer.protocol = 'tscp';
			peer.group    = $("#adl-val-select_group").val();
			peer.name     = $("#adl-val-select_device").val();
			break;
	}
	return peer;
}

// Process data before handing over to jqGrid
function adl_processPeer(peer) {
	// If an alias exists, use that as the name the user sees
	// We only need this for history, for phonebook each entry has unique alias
	peer.label = peer.alias || peer.name;

	// Map status codes to text
	var html = "";
	switch (peer.status) {
		case 0:
			if (peer.protocol==="tscp") {
				html = "?";
			}
			break;
		case 1:
			html = "<span style='font-weight: bold; color: red;'>down</span>";
			break;
		case 2:
			html = "<span style='font-weight: bold; color: green;'>up</span>";
			break;
	}
	peer.status = html;
}

function adl_beforeInitPeer(form) {
	var alias    = form.find("#alias, #peer\\.alias");
	var proto    = form.find("#protocol, #peer\\.protocol");
	var nameText = form.find("#tr_name, #tr_peer\\.name").find(".CaptionTD");

	// If no protocol is specified and entry does not exist in phonebook,
	// set protocol to TSCP (default).
	if (!alias.val() && !proto.val()) {
		proto.val("tscp");
	}

	// Add event handler to show/hide fields based on selected protocol
	proto.off().change(function(event, init) {
		var animationTime = (init===true) ? 0 : 400;
		var groupFields = form.find("#tr_group, #tr_grpwd, #tr_peer\\.group, #tr_peer\\.grpwd");
		switch ($(this).val()) {
			case "tscp":
				nameText.text("Name");
				groupFields.show(animationTime);
				break;
			case "tscpd":
			case "rtp":
			case "sip":
			default:
				nameText.text("Address");
				groupFields.hide(animationTime);
				break;
		}
	});
	// Upon opening form, hide/show fields with no animation
	proto.trigger('change', [true]);

	// Make checkbox to allow saving with current codec settings
	var cbox = $('<input type="checkbox" name="codec">').addClass("adl-custom_input");

	// Append checkbox to var table
	var table = form.find(".EditTable");
	var tr = $('<tr>').addClass("FormData").appendTo(table);
	$('<td>').addClass("CaptionTD").text("Save with current codec").appendTo(tr);
	$('<td>').addClass("DataTD").html("&nbsp;").append(cbox).appendTo(tr);

	// Make checkbox look good
	cbox.checkbox();

	return true;
}

function adl_beforeSubmitPeer(postdata, form) {
	// Remove everything in param name before a '.' character
	for (var i in postdata) {
		var res = i.replace(/^.*\./g, "");
		if (res != i) {
			postdata[res] = postdata[i];
			delete postdata[i];
			// If we are here we are submitting from history
			postdata.history = "true";
		}
	}

	// Check form for errors
	var error;
	if (!postdata.group && postdata.protocol=="tscp") error = "Must fill in 'Group'";
	if (error) return [false, "ERROR: " + error];

	// If no alias, default to using name/address
	if (!postdata.alias) {
		postdata.alias = postdata.name;
	}

	// Add any custom input fields to postdata (filter out unchecked checkboxes)
	form.find(".adl-custom_input").not("input:checkbox:not(:checked)").each(function(index) {
		var name = $(this).attr('name');
		var val  = $(this).val();
		postdata[name] = val;
		//console.log("postdata[" + name + "]=" + val);
	});

	// Make 'addr' variable from other fields
	postdata.addr = adl_makeAddr(postdata);

	// Delete what is now part of 'addr'
	delete postdata.name;
	delete postdata.protocol;
	delete postdata.port;
	delete postdata.group;
	delete postdata.server;
	return adl_beforeSubmitGeneric(postdata, form);
}

function adl_beforeSubmitGeneric(postdata, form) {
	return [true, "Waiting for response..."];
}

function adl_afterSubmitGeneric(response, postdata) {
	if (response.responseText != "OK") {
		console.log(response);
		return [false, response.responseText];
	}
	return [true, "Changes successful!"];
}

// Return string to use for dialing from peer object
function adl_makeAddr(peer) {
	var addr = "";
	if (peer.protocol) addr += peer.protocol + ":";
	if (peer.name)     addr += peer.name;
	if (peer.port)     addr += ":" + peer.port;
	if (peer.group)    addr += "@" + peer.group;
	if (peer.server)   addr += "@" + peer.server;
	return addr;
}
