// ©2014 The Telos Alliance. All Rights Reserved.

// common.js: Javascript functions common to multiple pages
// author:    Jeff Bencin <jeff.bencin@gmail.com>
//
// Javascript namespace: com_*
// CSS namespace: com-* (CSS in file common.css)

// Parse data from json/bitrates and return array of all available bitrates (in Kbps)
function com_parseBitratesFromJSON(data) {
	// Build a sorted list of unique bitrates for both channel modes
	var array = new Array();

	for (var mode in data) {
		var codecs = data[mode];
		for (var i in codecs) {
			var bitrates = codecs[i].bitrates;
			for (var j in bitrates) {
				var br = bitrates[j];
				var isInArray = array.indexOf(br);
				// Only add unique values
				if (isInArray < 0) {
					array.push(br);
				}
			}
		}
	}
	array.sort(function(a,b) { return a-b });

	return array;
}

// Select object with data-value==value, deselect others
// Show and hide objects if button has associated "display" variable
// For use with custom controls
function com_selectByValue(jqObj, value) {
	var selected;
	var show = $();
	var hide = $();
	jqObj.each(function() {
		var val = $(this).data("value");
		var display = $(jQuery(this).data("display"));
		if (val==value) {
			selected = $(this).addClass("com-selected");
			show = $(show).add(display);
		} else {
			$(this).removeClass("com-selected");
			hide = $(hide).add(display);
		}
	});
	$(hide).not(show).hide();
	$(show).show();
	return selected;
}

// Same as com_displayByValue() but for html forms
function com_htmlSelectByValue(jqObj, value) {
	var selected;
	var show = $();
	var hide = $();
	jqObj.each(function() {
		var val = $(this).val();
		var display = $(jQuery(this).data("display"));
		if (val==value) {
			selected = $(this);
			show = $(show).add(display);
		} else {
			hide = $(hide).add(display);
		}
	});
	$(hide).not(show).hide();
	$(show).show();
	return selected;
}

var com_ajaxSubscriptions = new Array();

// Allows functions from multiple files to register to listen to an AJAX response
// Uses lowest interval passed by any subscriber
function com_ajaxSubscribe(url, ival, callback) {
	var source = com_ajaxSubscriptions[url] || new Object();

	if (!source.ival || source.ival>ival) {
		source.ival = ival;
	}

	var clients = source.clients || new Array();
	var client = new Object();
	client.callback = callback;
	client.ival = ival; // Might be used in the future...
	clients.push(client);

	source.clients = clients;
	com_ajaxSubscriptions[url] = source;

	function makeRequest() {
		var args = {
			sid: Date.now() // Timestamp URL to avoid caching
		};
		var uniqueURL = com_appendGETParams(url, args);
		$.getJSON(uniqueURL, function(data) {
			for (c in clients) {
				clients[c].callback(data);
			}
		});
	}

	clearInterval(source.timer);
	makeRequest();
	source.timer = setInterval(makeRequest, source.ival);
}

// Append an object of name/value pairs to a URL as GET params
function com_appendGETParams(url, args, filter) {
	var sep = '?';
	for (var i in args) {
		url += sep + encodeURIComponent(i) + "=" + encodeURIComponent(args[i]);
		sep = '&';
	}
	return url;
}
