// ©2014 The Telos Alliance. All Rights Reserved.

// ajax.js: Common code for ajax requests
// author:  Jeff Bencin <jeff.bencin@gmail.com>
//
// Javascript namespace: Z1ajax_*

var Z1ajax_disable = false; // This is meant to be set via console for debugging
var Z1ajax_xmlHttpObj;

var Z1ajax_updateId;
var Z1ajax_messageId;
var Z1ajax_URL;
var Z1ajax_interval;
var Z1ajax_delay = new Object();

// updateId: Id of element to be updated 
// messageId: Id of element where status messages will be written (can be NULL)
// url: URL for update request
// ival: Interval, in milliseconds, of updates. If '1', will not repeat. DO NOT USE '0'.
// delay: delay between message "Loading..." and sending request. Let's the user see it's working.
function Z1ajax_init(updateId, messageId, url, ival, delay)
{
	Z1ajax_updateId = updateId;
	Z1ajax_messageId = messageId;
	Z1ajax_URL = url;
	Z1ajax_interval = ival;
	Z1ajax_delay.val = delay;

	Z1ajax_xmlHttpObj = new XMLHttpRequest();

	setTimeout(Z1ajax_send, Z1ajax_interval);
};

function Z1ajax_send()
{
	if (Z1ajax_disable) return;
	Z1ajax_setMessage("Loading...", "green");

	if (Z1ajax_delay.val) {
		if (!Z1ajax_delay.done) {
			Z1ajax_delay.done = true;
			setTimeout(Z1ajax_send, Z1ajax_delay.val);
			return;
		}
		Z1ajax_delay.done = false;
	}

	// sUID prevents browser caching
	Z1ajax_xmlHttpObj.open("GET", Z1ajax_URL + "&sUID=" + new Date().getTime(), true);
	Z1ajax_xmlHttpObj.onreadystatechange = Z1ajax_receive;
	Z1ajax_xmlHttpObj.send(null);
};

function Z1ajax_receive()
{
    if(Z1ajax_xmlHttpObj.readyState!=4) return; // if request is not complete

    var bComplete = false;

    while (!bComplete) {
        if (Z1ajax_xmlHttpObj.status!=200) break; // if error

        var response = Z1ajax_xmlHttpObj.responseText;
        if (!response) break;

        Z1ajax_setMessage("&nbsp;", "green");
        Z1ajax_update(response);
        bComplete = true;
    }

    if (!bComplete) {
        Z1ajax_setMessage("Error! Could not communicate with server!", "red");
    }

	if (Z1ajax_interval >= 2) {
    	setTimeout(Z1ajax_send, Z1ajax_interval);
	}
};

// TODO: Should parse target id out of HTML comments instead.
//       Would allow updates of separate sections in one request.
function Z1ajax_update(response)
{
	document.getElementById(Z1ajax_updateId).innerHTML = response;
};

function Z1ajax_setMessage (msg, color)
{
    var element = document.getElementById(Z1ajax_messageId);
    if (element)
    {
        element.innerHTML   = msg;
        element.style.color = color;
    }
};
