// ©2014 The Telos Alliance. All Rights Reserved.

// validate.js: Code to validate HTML forms
// author:      Jeff Bencin <jeff.bencin@gmail.com>
//
// Javascript namespace: validate*

var g_errors = new Array(); 
var g_formValidators = new Array();
var g_ipFmt = /^\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}$/; 
var g_validSubnet = /255|254|252|248|240|224|192|128|0/;

function validateIPv4Address(ip) 
{
	// Test format
	if (!g_ipFmt.test(ip)) return "Invalid format";

	var parts = ip.split(".");
	
	// If any part is greater than 255
	for (var i=0; i<parts.length; i++) {
		if (parseInt(parts[i]) > 255){
			return "Octet " + (i+1) + " too big";
		}
	}

	return null;
};

function validateIP(obj, formName, msgArea) 
{
	var ip = obj.value = obj.value.replace( /\s/g, ""); // Remove spaces
	var msg = validateIPv4Address(ip);
	updatePage(msg, formName, msgArea);
	return msg;
}

function validatePort(obj, formName, msgArea) 
{
	var port = obj.value = parseInt(obj.value.replace( /\s/g, "")) || 0;
	var msg;

	if (port < 0 || port > 65535) {
		msg = "Must be 0-65535";
	}
	updatePage(msg, formName, msgArea);
	return msg;
}

function validateNetmask(obj, formName, msgArea) 
{
	var mask = obj.value = obj.value.replace( /\s/g, ""); // Remove spaces
	var msg;

	do {
		if (!g_ipFmt.test(mask)) {
			msg = "Invalid format";
			break;
		}

		var parts = mask.split(".");
		var restIsZeros = false;

		for (var i=0; i<parts.length; i++) {
			var val = parts[i];
			if (!g_validSubnet.test(val)) {
				msg = "Octet " + (i+1) + " invalid";
				break;
			}
			if (val != "0" && restIsZeros) {
				msg = "Octet " + (i+1) + " invalid";
				break;
			}
			else if (val != "255") {
				restIsZeros = true;
			}
		}
	} while (0);

	updatePage(msg, formName, msgArea);
	return msg;
};

function validateAllIPSettings(ip, mask, gw, dns)
{
	//return "vars: " + ip + "," + mask + "," + gw + "," + dns;

	// Interface disabled. We don't care about validating anything.
	if (ip=="0.0.0.0") return null;

	// Assume we have already checked IP addresses and Mask is valid.
	// Here we will see it they all work together.

	var pIP   = ip.split(".");
	var pMask = mask.split(".");
	var pGway = gw.split(".");
	var pDNS  = dns.split(".");

	if (gw!="0.0.0.0") {
		// Check if we are in same subnet as Gateway
		for (var i=0; i<4; i++) {
			if ((parseInt(pIP[i]) ^ parseInt(pGway[i])) & parseInt(pMask[i])) {
				return "Gateway is outside of subnet";
			}
		}
	}
	else if (dns!="0.0.0.0") {
		// If gateway is empty and DNS is not, check if reachable in subnet
		for (var i=0; i<4; i++) {
			if ((parseInt(pIP[i]) ^ parseInt(pDNS[i])) & parseInt(pMask[i])) {
				return "DNS server is not reachable";
			}
		}
	}
	
	return null;
}

function updatePage(msg, formName, msgArea)
{
	var elem = document.getElementById(msgArea);
	if (elem) {
		if (!g_errors[formName]) {
			g_errors[formName] = new Array();
		}
		if (msg) {
			msg += "!&nbsp;";
			elem.innerHTML = msg;
			elem.style.color = "red";
			g_errors[formName][msgArea] = msg;
		}
		else {
			elem.innerHTML = "";
			delete g_errors[formName][msgArea];
		}
	}
};

// Function should return null or an array in the form [int statusCode, string message]
// null: No error or message
// statusCode 0: No error, but display message in alert
// statusCode 1: Warning, let user decide whether to continue
// statusCode 2: Error, do not allow form submission
function setFormValidator(formName, validator)
{
	g_formValidators[formName] = validator;
}

// Assign as "onsubmit" for all forms, like this: 
//   <form onsubmit='return !validateForm || validateForm(this);'>
// It is completely safe to use this way
function validateForm(formObj) 
{
	// Check for any errors from other validation functions
	var formName = formObj.name;
	for (errors in g_errors[formName]) {
		alert("You must fix errors before you can submit!");
		return false;
	}

	// Check if we have made an additional validator for this form 
	var validator = g_formValidators[formName];
	if (!validator) return true;
	var msg = validator();
	if (!msg || !msg[0]) return true;
	switch (msg[0]) {
		case 0:
			alert(msg[1]);
			return true;
		case 1:
			return confirm(msg[1]);
		case 2:
			alert(msg[1]);
			return false;
	}
	return false;
}
